/******************************************************************************
 *
 *	Tic Tac Toe V0.90 (c) David A.S.Crofts
 *
 *	23 Brisbane Street
 *	Berwick Victoria 3806
 *	Australia
 *
 *	email: dasc1961@netscape.net
 *
 *****************************************************************************/

#include <PalmOS.h>

#include "tic_tac_toe.h"
#include "tic_tac_toe_rc.h"


/***********************************************************************
 *
 *	Internal Constants
 *
 ***********************************************************************/

#define BitmapSquareSize		32

#define P1BoardOffsetX			17
#define P1BoardOffsetY			17

#define P1BoardGridWidth		5
#define P1BoardSquareSize		38

#define P2BoardOffsetX			17
#define P2BoardOffsetY			17

#define P2BoardGridWidth		5
#define P2BoardSquareSize		38


/***********************************************************************
 *
 *	Private global variables
 *
 ***********************************************************************/

static Int16 BoardOffsetX =		P1BoardOffsetX;
static Int16 BoardOffsetY =		P1BoardOffsetY;

static Int16 BoardGridWidth =		P1BoardGridWidth;
static Int16 BoardSquareSize =		P1BoardSquareSize;


static GameType		Game;
static TTTPrefType	TTTPref;

static WinHandle	OffscreenBoardWinH = 0;
static WinHandle	OffscreenBitmapWinH = 0;

static Int16		BitmapTable[] = {
    P1EmptyBitmap,
    P1CrossBitmap,
    P1NoughtBitmap,
    P2EmptyBitmap,
    P2CrossBitmap,
    P2NoughtBitmap
};

static STATE		move;

static STATE_PTR	FreeStates = NULL;
static SUCESSOR_PTR	FreeSucessors = NULL;


/***********************************************************************
 *
 *	Internal Functions
 *
 ***********************************************************************/

static UInt16 	StartApplication(void);
static void 	StopApplication(void);
static Boolean 	MainFormOneDoCommand(UInt16 command);
static Boolean  MainFormTwoDoCommand (UInt16 command);
static Boolean 	MainFormOneHandleEvent(EventPtr event);
static Boolean	MainFormTwoHandleEvent(EventPtr event);
static Boolean 	CurrentFormHandleEvent(EventPtr event);

static void 	DrawGameBoard(void);
static void 	UpdateStatusDisplay(void);
static void     DrawStatusBar(Int16 x, Int16 o, Int16 d);
static Boolean 	HandlePenDown(
    Int16 penX, Int16 penY, Boolean marking, Boolean * inBoundsP);
static Boolean 	MapPenPosition(
    Int16 penX, Int16 penY, PieceCoordType* coordP );
static void 	DrawPiece(WinHandle dstWinH, Int16 row, Int16 col);
static void 	DrawBitmap(
    WinHandle winHandle, Int16 resID, Int16 x, Int16 y);
static void 	GetPieceRectangle(Int16 row, Int16 col, RectanglePtr rP);
static void 	NewGameBoard(void);
static void 	SaveGameBoard(void);
static void 	LoadGameBoard(void);
static Int16	ConvertSquare(UInt16 bitmap);
static Int16	ConvertBitmap(UInt16 piece);
static void	CaptureGraphics(void);
static void	RestoreGraphics(void);
static void 	DrawGrid(void);

SUCESSOR_PTR	play(STATE_PTR p, UInt16 depth);
Int16		f(STATE_PTR p, Int16 alpha, Int16 beta, UInt16 depth);
SUCESSOR_PTR	gen_sucessors(STATE_PTR p);
void 		free_sucessors(SUCESSOR_PTR q);
Int16 		value(STATE_PTR p);
STATE_PTR 	New_STATE(void);
SUCESSOR_PTR 	New_SUCESSOR(void);
void 		copy_state(STATE_PTR p, STATE_PTR q);


/***********************************************************************
 *
 * FUNCTION:     NewGameBoard
 *
 ***********************************************************************/

static void NewGameBoard(void)
{
    UInt16	i, bitmapID;
	
    // Initialize the board
    
    if (Game.numPlayers != 2) {

	Game.P1state = gameInProgress;

        Game.P1value = 0;
    
        for ( i = 0; i < 9; ++i ) 
	    Game.P1square[i] = P1EmptyBitmap;
        
    } else {

        Game.P2state = gameInProgress;

        Game.P2value = 0;
    
        for ( i = 0; i < 9; ++i ) 
	    Game.P2square[i] = P2EmptyBitmap;
    }
}


/***********************************************************************
 *
 * FUNCTION:     LoadGameBoard
 *
 ***********************************************************************/

static void LoadGameBoard(void)
{
    Boolean	found;
	
    // Try loading a saved game board - if that fails, generate a new one

    found = PrefGetAppPreferencesV10(
        tttAppCreator, tttAppVersionNum,
	&TTTPref, sizeof(TTTPref) );

    if ( found && TTTPref.signature == TTTPrefSignature )
	Game = TTTPref.game;
    else {
	Game.numPlayers = 2;
	NewGameBoard();

	Game.numPlayers = 1;
	NewGameBoard();

	Game.P1reversed = false;
	Game.P1turn = COMPUTER_PIECE;
	Game.P1Draws = 0;
	Game.P1Loses = 0;

        Game.P2IsFirstMove = true;
        Game.P2FirstMove = PLAYER_PIECE;
	Game.P2turn = PLAYER_PIECE;	
        Game.P2NoughtWins = 0;
        Game.P2CrossWins = 0;
        Game.P2Draws = 0;
    }
}


/***********************************************************************
 *
 * FUNCTION:     SaveGameBoard
 *
 ***********************************************************************/

static void SaveGameBoard(void)
{
    Int16 i;

    TTTPref.signature = TTTPrefSignature;
    TTTPref.game = Game;

    // Save our preferences to the Preferences database

    PrefSetAppPreferencesV10( 
    tttAppCreator, 
    tttAppVersionNum, 
    &TTTPref,
    sizeof(TTTPref) );
}


/***********************************************************************
 *
 * FUNCTION:     HandlePenDown
 *
 ***********************************************************************/

static Boolean HandlePenDown(
    Int16 penX, Int16 penY, Boolean marking, Boolean * inBoundsP)
{
    PieceCoordType	coord;
    Int16		i, j, pieces;
    SUCESSOR_PTR	q;

    *inBoundsP = MapPenPosition( penX, penY, &coord );

    if ( !(*inBoundsP)) {
        
	return( false );
    }

    i = (coord.row)*3+coord.col;

    if (Game.numPlayers != 2) {

        if (Game.P1square[i] == P1EmptyBitmap) {

	    CaptureGraphics();

	    if (value(&move) == 0) {

                if (!Game.P1reversed)
	    	    Game.P1square[i] = P1CrossBitmap;	
	        else
	    	    Game.P1square[i] = P1NoughtBitmap;	

                DrawPiece(0, coord.row, coord.col);
	    
	        CaptureGraphics();

	        move.turn = COMPUTER_PIECE;
	        q = play( &move, 10 );
	
	        if (q->num > 0) {
	            copy_state(
	            q->ptr[(UInt16)SysRandom( 0 ) % q->num], &move);
	            Game.P1value = -move.piece;
	        }
                free_sucessors(q);
	        if (value(&move) != 0) {
		    Game.P1state = gameLost;
		    Game.P1Loses++;
		} else {
		    q = gen_sucessors(&move);
		    if (q->num == 0) {
		        Game.P1state = gameDraw;
			Game.P1Draws++;
		    }
		    free_sucessors(q);
	        }

                RestoreGraphics();
	        DrawGameBoard();
            }
        }

    } else {
     
        if (Game.P2square[i] == P2EmptyBitmap) {

            if (Game.P2IsFirstMove) {
	        Game.P2turn = Game.P2FirstMove;	
                Game.P2IsFirstMove = false;
	    }
	    
	    CaptureGraphics();

	    if (value(&move) == 0) {

                if (Game.P2turn == COMPUTER_PIECE){
		    Game.P2turn = PLAYER_PIECE;
	    	    Game.P2square[i] = P2NoughtBitmap;	
	        } else {
		    Game.P2turn = COMPUTER_PIECE;
	    	    Game.P2square[i] = P2CrossBitmap;
		}

                DrawPiece(0, coord.row, coord.col);
	    
	        CaptureGraphics();

		move.turn = Game.P2turn;
		
	        if (Game.P2value == 0) {
		    q = gen_sucessors(&move);
	            if (q->num == 0) {
		        Game.P2state = gameDraw;
			Game.P2Draws++;
		    }
		    free_sucessors(q);
		}

		for (q=NULL, pieces=0, j=0; j<9; ++j) {
		    if (
		    (Game.P2square[j] == P2NoughtBitmap) ||
		    (Game.P2square[j] == P2CrossBitmap)) {
			if ( ++pieces > 1) {
	                    q = play( &move, 10 );
			    break;
		        }
		    }
	        }
		
		    
	        if (q != NULL && q->num > 0) {
	            Game.P2value = q->ptr[0]->piece;
		    if (Game.P2turn == COMPUTER_PIECE)
		        Game.P2value = -Game.P2value;
	        }
		
		if (q != NULL && q->num == 0) {
		    if (Game.P2value > 0) {
		        Game.P2CrossWins++;
		    } else if (Game.P2value < 0) {
		        Game.P2NoughtWins++;
		    }
		}

                if (q != NULL)
		    free_sucessors(q);
            }
            UpdateStatusDisplay();
        }
    }
    return( true );
}


/***********************************************************************
 *
 * FUNCTION:     MapPenPosition
 *
 ***********************************************************************/

static Boolean MapPenPosition(Int16 penX, Int16 penY,
    PieceCoordType* coordP )
{
    Int16		x;
    Int16		y;
    RectangleType	rect;
	
    // Map display relative coordinates to window-relative

    x = (Int16)penX;
    y = (Int16)penY;

    WinDisplayToWindowPt( &x, &y );
	
    rect.topLeft.x = BoardOffsetX;
    rect.topLeft.y = BoardOffsetY;
    rect.extent.x = 3 * BoardSquareSize + 2 * BoardGridWidth;
    rect.extent.y = 3 * BoardSquareSize + 2 * BoardGridWidth;
	
    if ( !RctPtInRectangle(x, y, &rect) )
	return( false );
	
    // Convert to board-relative coordinates

    x -= BoardOffsetX;
    y -= BoardOffsetY;
	
    if ( x < 0 ) {
	ErrDisplay( "board x is negative" );
	x = 0;
    }

    if ( y < 0 ) {
	ErrDisplay( "board y is negative" );
	y = 0;
    }
	
    // Get the piece position

    if (x < BoardSquareSize)
        coordP->col = 0;
    else if (
    (x > (1*BoardSquareSize + BoardGridWidth)) &&
    (x < (2*BoardSquareSize + BoardGridWidth)))
        coordP->col = 1;
    else if (
    (x > (2*BoardSquareSize + 2*BoardGridWidth)) &&
    (x < (3*BoardSquareSize + 2*BoardGridWidth)))
        coordP->col = 2;
    else
        return( false );

    if (y < BoardSquareSize)
        coordP->row = 0;
    else if (
    (y > (1*BoardSquareSize + BoardGridWidth)) &&
    (y < (2*BoardSquareSize + BoardGridWidth)))
        coordP->row = 1;
    else if (
    (y > (2*BoardSquareSize + 2*BoardGridWidth)) &&
    (y < (3*BoardSquareSize + 2*BoardGridWidth)))
        coordP->row = 2;
    else
        return( false );

    return( true );
}


/***********************************************************************
 *
 * FUNCTION:     DrawPiece
 *
 ***********************************************************************/

static void DrawPiece(WinHandle dstWinH, Int16 row, Int16 col)
{
    RectangleType	rect;

    Int16	bitmapID = 0;
    Int16	bitmapIndex = 0;

    if (Game.numPlayers == 2)
	bitmapID = Game.P2square[ row*3+col ];
    else
	bitmapID = Game.P1square[ row*3+col ];

    bitmapIndex = bitmapID - 1000;
	
    // Get bounds of the game piece

    GetPieceRectangle( row, col, &rect );

    // Draw the bitmap

    // Optimization

    if ( OffscreenBitmapWinH ) {
	
	RectangleType	srcRect;

	srcRect.topLeft.x = BitmapSquareSize * bitmapIndex;
	srcRect.topLeft.y = 0;
	srcRect.extent.x = BitmapSquareSize;
	srcRect.extent.y = BitmapSquareSize;
	WinCopyRectangle ( 
	    OffscreenBitmapWinH, dstWinH, &srcRect,
	    rect.topLeft.x, rect.topLeft.y, winPaint 
	);

    } else
		
	DrawBitmap( dstWinH, bitmapID, 
	    rect.topLeft.x, 
	    rect.topLeft.y 
	);
}


/***********************************************************************
 *
 * FUNCTION:	DrawBitmap
 *
 ***********************************************************************/

static void DrawBitmap(WinHandle winHandle, Int16 resID, Int16 x, Int16 y)
{
    MemHandle	resH;
    BitmapPtr	resP;
    WinHandle	currDrawWindow;
	
    // If passed a non-null window MemHandle, 
    // set it up as the draw window, saving the current draw window

    if ( winHandle )
	currDrawWindow = WinSetDrawWindow( winHandle );
		
    resH = DmGetResource( bitmapRsc, resID );
    ErrFatalDisplayIf( !resH, "no bitmap" );
    resP = MemHandleLock(resH);
    WinDrawBitmap (resP, x, y);
    MemPtrUnlock(resP);
    DmReleaseResource( resH );
	
    // Restore the current draw window

    if ( winHandle )
	WinSetDrawWindow( currDrawWindow );
}


/***********************************************************************
 *
 * FUNCTION:     GetPieceRectangle
 *
 ***********************************************************************/

static void GetPieceRectangle(Int16 row, Int16 col, RectanglePtr rP)
{
    Int16 BitmapOffset;

    ErrFatalDisplayIf( row >= 3, "bad row" );
    ErrFatalDisplayIf( col >= 3, "bad column" );
    ErrFatalDisplayIf( !rP, "bad param" );
    
    BitmapOffset = (BoardSquareSize - BitmapSquareSize)/2+1;
	
    ErrFatalDisplayIf( BitmapOffset < 0, "bitmap bigger then square" );

    switch (col) {
    case 0:
        rP->topLeft.x = 
	BoardOffsetX + BitmapOffset;
	break;
    case 1:
        rP->topLeft.x = 
	BoardOffsetX + BoardSquareSize + BoardGridWidth + BitmapOffset;
	break;
    case 2:
        rP->topLeft.x = 
	BoardOffsetX + 2*BoardSquareSize + 2*BoardGridWidth + BitmapOffset;
	break;
    }
    
    switch (row) {
    case 0:
        rP->topLeft.y = 
	BoardOffsetY + BitmapOffset;
	break;
    case 1:
        rP->topLeft.y = 
	BoardOffsetY + BoardSquareSize + BoardGridWidth + BitmapOffset;
	break;
    case 2:
        rP->topLeft.y = 
	BoardOffsetY + 2*BoardSquareSize + 2*BoardGridWidth + BitmapOffset;
	break;
    }
    
    rP->extent.x = BitmapSquareSize;
    rP->extent.y = BitmapSquareSize;
}


/***********************************************************************
 *
 * FUNCTION:     DrawGameBoard
 *
 ***********************************************************************/

static void DrawGameBoard(void)
{
    Int16	i, j;

    for ( i = 0; i < 3; i++ )
	for ( j = 0; j < 3; j++ )
	    DrawPiece( OffscreenBoardWinH, i, j );
	
    if ( OffscreenBoardWinH ) {

    	RectangleType	srcRect;
		
	srcRect.topLeft.x = BoardOffsetX;
	srcRect.topLeft.y = BoardOffsetY;
	srcRect.extent.x = 3 * BoardSquareSize + 2 * BoardGridWidth + 1;
	srcRect.extent.y = 3 * BoardSquareSize + 2 * BoardGridWidth + 1;
	
	WinCopyRectangle ( 
	    OffscreenBoardWinH /*srcWin*/, 
	    0 /*dstWin*/, 
	    &srcRect,
	    BoardOffsetX, BoardOffsetY, winPaint );
    }

    UpdateStatusDisplay();

}


/***********************************************************************
 *
 * FUNCTION:     UpdateStatusDisplay
 *
 ***********************************************************************/

static void UpdateStatusDisplay(void)
{
    RectangleType	r;
    Int32		value = 0;

    Char		buf[80];
    Char		txt1[32] = "X wins = ";
    Char		txt2[32] = "O wins = ";
    Char *		cP;
    UInt16		txtLen;

    r.topLeft.x = 77;
    r.topLeft.y = 0;
    r.extent.x = 53;
    r.extent.y = 13;
    WinEraseRectangle( &r, 0 );
	
    if (Game.numPlayers != 2) {

        value = Game.P1value;

        if (Game.P1state == gameDraw) {
	    WinDrawChars( " Draw!!!", 8, 77, 0);
        } else if ( value > 10000 ) {
	    WinDrawChars( " You Win!!!", 11, 77, 0);
        } else if ( value < -10000 ) {
	    WinDrawChars( " I Win!!!", 9, 77, 0);
        } else if ( Game.P1reversed ) {
	    WinDrawChars( "You're O ", 9, 77, 0);
	} else {
	    WinDrawChars( "You're X ", 9, 77, 0);
	}

 	if (Game.P1reversed)
            DrawStatusBar(Game.P1Loses, 0, Game.P1Draws);
	else
            DrawStatusBar(0, Game.P1Loses, Game.P1Draws);
	    
    } else {

        value = Game.P2value;

        if (Game.P2state == gameDraw)
	    WinDrawChars( "Draw!!!   ", 10, 77, 0);
        else if ( value > 10000 )
	    WinDrawChars( "X wins!!! ", 10, 77, 0);
        else if ( value < -10000 )
	    WinDrawChars( "O wins!!! ", 10, 77, 0);
        else if (Game.P2turn == PLAYER_PIECE)
            WinDrawChars( "X to move ", 10, 77, 0);
	else
	    WinDrawChars( "O to move ", 10, 77, 0);
	
        DrawStatusBar(Game.P2CrossWins, Game.P2NoughtWins, Game.P2Draws);
    }
}


/***********************************************************************
 *
 * FUNCTION:     DrawStatusBar
 *
 ***********************************************************************/

static void DrawStatusBar(Int16 x, Int16 o, Int16 d)
{
    RectangleType	r;
    Char		buf[80];
    Char *		cP;
    UInt16		txtLen;

    r.topLeft.x = 0;
    r.topLeft.y = 147;
    r.extent.x = 160;
    r.extent.y = 13;
    WinEraseRectangle( &r, 0 );

    WinDrawLine( 0, 145, 159, 145 );
    WinDrawLine( 0, 146, 0, 159 );
    WinDrawLine( 159, 146, 159, 159 );
    WinDrawLine( 0, 159, 159, 159 );

    cP = buf;
    StrCopy(cP, "X wins = ");
    txtLen = StrLen( buf );
    cP = buf+txtLen;
    StrIToA( cP, x );
    txtLen = StrLen( buf );
    cP = buf+txtLen;
    StrCopy(cP, ", O wins = ");
    txtLen = StrLen( buf );
    cP = buf+txtLen;
    StrIToA( cP, o );
    txtLen = StrLen( buf );
    cP = buf+txtLen;
    StrCopy(cP, ", Draws = ");
    txtLen = StrLen( buf );
    cP = buf+txtLen;
    StrIToA( cP, d );
    txtLen = StrLen( buf );
    WinDrawChars( buf, txtLen, 4, 147);
}


/***********************************************************************
 *
 * FUNCTION:     StartApplication
 *
 ***********************************************************************/

static UInt16 StartApplication(void)
{
    UInt16	error;
    Int32	i;
    RectangleType	r;

    r.topLeft.x = 0;
    r.topLeft.y = 0;
    r.extent.x = 159;
    r.extent.y = 159;
    WinEraseRectangle( &r, 0 );
	
    SysRandom( TimGetSeconds() );

    LoadGameBoard();
	
    // Initialize offscreen bitmap window once (used for drawing optimization -
    // this significantly speeds up mass redraws such as during during initial
    // draw, board restoraration, and screen update)

    if ( !OffscreenBitmapWinH ) {
	Int16	i;

	OffscreenBitmapWinH = WinCreateOffscreenWindow(
	BitmapSquareSize * 6, BitmapSquareSize, 
	screenFormat, &error );

	if ( OffscreenBitmapWinH ) {
	    for ( i = 0; i < 6; i++ ) {
	    	DrawBitmap( 
		OffscreenBitmapWinH, 
		BitmapTable[i], 
		BitmapSquareSize * i,
		0 );
	    }
	}
    }
	
    if (Game.numPlayers == 2) {
	BoardOffsetX =    P2BoardOffsetX;
	BoardOffsetY =    P2BoardOffsetY;
	BoardGridWidth =  P2BoardGridWidth;
        BoardSquareSize = P2BoardSquareSize;
    }
	
    if ( !OffscreenBoardWinH ) {
	
        WinHandle	currDrawWindow;
        RectangleType	r;
		
	OffscreenBoardWinH = WinCreateOffscreenWindow(
	BoardOffsetX + 3 * BoardSquareSize + 2 * BoardGridWidth + 1, 
	BoardOffsetY + 3 * BoardSquareSize + 2 * BoardGridWidth + 1,
	screenFormat, &error );

        if ( OffscreenBoardWinH )
	    currDrawWindow = WinSetDrawWindow( OffscreenBoardWinH );

	
    	r.topLeft.x = BoardOffsetX;
    	r.topLeft.y = BoardOffsetY;
    	r.extent.x = 3 * BoardSquareSize + 2 * BoardGridWidth + 1;
    	r.extent.y = 3 * BoardSquareSize + 2 * BoardGridWidth + 1;
    	WinEraseRectangle( &r, 0 );
	
        DrawGrid();

        if ( OffscreenBoardWinH )
    	    WinSetDrawWindow( currDrawWindow );
    }

    if (Game.numPlayers == 2)
        FrmGotoForm(MainFormTwo);
    else
        FrmGotoForm(MainFormOne);

    DrawGameBoard();
    return( 0 );
}


/***********************************************************************
 *
 * FUNCTION:	StopApplication
 *
 ***********************************************************************/

static void StopApplication (void)
{
    STATE_PTR p;
    SUCESSOR_PTR q;

    while (FreeStates != NULL) {
        p = FreeStates;
	FreeStates = FreeStates->memPtr;
	MemHandleFree(p->memHandle);
    }

    while (FreeSucessors != NULL) {
        q = FreeSucessors;
	FreeSucessors = FreeSucessors->memPtr;
	MemHandleFree(q->memHandle);
    }

    SaveGameBoard();

    FrmCloseAllForms ();
	
    // Free the offscreen window

    if ( OffscreenBitmapWinH ) {
	WinDeleteWindow( OffscreenBitmapWinH, false );
	OffscreenBitmapWinH = 0;
    }

    // Free the offscreen window

    if ( OffscreenBoardWinH ) {
	WinDeleteWindow( OffscreenBoardWinH, false );
	OffscreenBoardWinH = 0;
    }
}


/***********************************************************************
 *
 * FUNCTION:    MainFormOneDoCommand
 *
 ***********************************************************************/

static Boolean MainFormOneDoCommand (UInt16 command)
{
    Boolean	handled = false;
    Int16	i;

    UInt16	error;
    WinHandle		currDrawWindow;
    RectangleType	r;

    MenuEraseStatus( MenuGetActiveMenu() );

    switch (command){
    
    case MainOptionsOneAboutCmd:

        FrmAlert( MainOptionAbout );
	handled = true;
	break;

    case MainOptionsOneResetCmd:

	Game.P1turn = COMPUTER_PIECE;
	Game.P1Draws = 0;
	Game.P1Loses = 0;
	
        NewGameBoard();	

        DrawGameBoard();

	handled = true;
	break;

    case MainOptionsOneBeXCmd:

        if (Game.P1reversed) {

            Game.P1reversed = false;

	    for (i = 0; i < 9; ++i) {
	        switch (Game.P1square[i]) {
		case P1CrossBitmap:
		    Game.P1square[i] = P1NoughtBitmap;
		    break;
		case P1NoughtBitmap:
		    Game.P1square[i] = P1CrossBitmap;
		    break;
		}
	    }
            DrawGameBoard();
	}
	handled = true;
	break;

    case MainOptionsOneBeOCmd:

        if (!Game.P1reversed) {

            Game.P1reversed = true;

	    for (i = 0; i < 9; ++i) {
	        switch (Game.P1square[i]) {
		case P1CrossBitmap:
		    Game.P1square[i] = P1NoughtBitmap;
		    break;
		case P1NoughtBitmap:
		    Game.P1square[i] = P1CrossBitmap;
		    break;
		}
	    }
            DrawGameBoard();
	}
	handled = true;
	break;

    case MainOptionsOneP2Cmd:

	BoardOffsetX =    P2BoardOffsetX;
	BoardOffsetY =    P2BoardOffsetY;
	BoardGridWidth =  P2BoardGridWidth;
        BoardSquareSize = P2BoardSquareSize;

        if ( OffscreenBoardWinH )
	    WinDeleteWindow( OffscreenBoardWinH, false );
		
	OffscreenBoardWinH = WinCreateOffscreenWindow(
	BoardOffsetX + 3 * BoardSquareSize + 2 * BoardGridWidth + 1, 
	BoardOffsetY + 3 * BoardSquareSize + 2 * BoardGridWidth + 1,
	screenFormat, &error );

        if ( OffscreenBoardWinH )
	    currDrawWindow = WinSetDrawWindow( OffscreenBoardWinH );

	r.topLeft.x = BoardOffsetX;
    	r.topLeft.y = BoardOffsetY;
    	r.extent.x = 3 * BoardSquareSize + 2 * BoardGridWidth + 1;
    	r.extent.y = 3 * BoardSquareSize + 2 * BoardGridWidth + 1;
    	WinEraseRectangle( &r, 0 );
	
        DrawGrid();

        if ( OffscreenBoardWinH )
    	    WinSetDrawWindow( currDrawWindow );

        FrmCloseAllForms();

	r.topLeft.x = 0;
    	r.topLeft.y = 0;
    	r.extent.x = 159;
    	r.extent.y = 159;
    	WinEraseRectangle( &r, 0 );
	
        FrmGotoForm(MainFormTwo);

        Game.numPlayers = 2;

        DrawGameBoard();

	handled = true;
	break;

    }	
    return handled;
}


/***********************************************************************
 *
 * FUNCTION:    MainFormTwoDoCommand
 *
 ***********************************************************************/

static Boolean MainFormTwoDoCommand (UInt16 command)
{
    Boolean	handled = false;
    Int16	i;

    UInt16		error;
    WinHandle		currDrawWindow;
    RectangleType	r;

    MenuEraseStatus( MenuGetActiveMenu() );

    switch (command){
    
    case MainOptionsTwoAboutCmd:

        FrmAlert( MainOptionAbout );
	handled = true;
	break;

    case MainOptionsTwoResetCmd:

        Game.P2IsFirstMove = true;
	Game.P2FirstMove = PLAYER_PIECE;	
	Game.P2turn = PLAYER_PIECE;	
	Game.P2NoughtWins = 0;
	Game.P2CrossWins = 0;
	Game.P2Draws = 0;
	
        NewGameBoard();	

        DrawGameBoard();

	handled = true;
	break;

    case MainOptionsTwoP1Cmd:

	BoardOffsetX =    P1BoardOffsetX;
	BoardOffsetY =    P1BoardOffsetY;
	BoardGridWidth =  P1BoardGridWidth;
        BoardSquareSize = P1BoardSquareSize;

        if ( OffscreenBoardWinH )
	    WinDeleteWindow( OffscreenBoardWinH, false );
		
	OffscreenBoardWinH = WinCreateOffscreenWindow(
	BoardOffsetX + 3 * BoardSquareSize + 2 * BoardGridWidth + 1, 
	BoardOffsetY + 3 * BoardSquareSize + 2 * BoardGridWidth + 1,
	screenFormat, &error );

        if ( OffscreenBoardWinH )
	    currDrawWindow = WinSetDrawWindow( OffscreenBoardWinH );

	r.topLeft.x = BoardOffsetX;
    	r.topLeft.y = BoardOffsetY;
    	r.extent.x = 3 * BoardSquareSize + 2 * BoardGridWidth + 1;
    	r.extent.y = 3 * BoardSquareSize + 2 * BoardGridWidth + 1;
    	WinEraseRectangle( &r, 0 );
	
        DrawGrid();

        if ( OffscreenBoardWinH )
    	    WinSetDrawWindow( currDrawWindow );

        FrmCloseAllForms();
	r.topLeft.x = 0;
    	r.topLeft.y = 0;
    	r.extent.x = 159;
    	r.extent.y = 159;
    	WinEraseRectangle( &r, 0 );
	
        FrmGotoForm(MainFormOne);

        Game.numPlayers = 1;
	
        DrawGameBoard();

	handled = true;
	break;
    }	
    return handled;
}


/***********************************************************************
 *
 * FUNCTION:    MainFormOneHandleEvent
 *
 ***********************************************************************/

static Boolean MainFormOneHandleEvent (EventPtr event)
{
    FormPtr frm;
    Boolean handled = false;
    UInt16 i;

    if (event->eType == ctlSelectEvent) {
	
	switch (event->data.ctlSelect.controlID) {

	case MainFormOneNewGameButton:

	    NewGameBoard();
	    if (Game.P1turn == COMPUTER_PIECE) {
	        Game.P1turn = PLAYER_PIECE;
                i = (UInt16)SysRandom( 0 ) % 9;
                if (!Game.P1reversed)
	    	    Game.P1square[i] = P1NoughtBitmap;	
	        else
	    	    Game.P1square[i] = P1CrossBitmap;
	    } else
	        Game.P1turn = COMPUTER_PIECE;
	    
            DrawGameBoard();
	    break;
			
	default:
	    break;
	}

    } else if ( event->eType == penDownEvent ) {

	Boolean	inBounds;
				
	handled = HandlePenDown( event->screenX, event->screenY,
	((KeyCurrentState() & (keyBitPageUp | keyBitPageDown)) != 0), 
	&inBounds );
		
	if ( !handled && inBounds ) {

	    handled = true;
	}
		
    } else if (event->eType == menuEvent) {

	return MainFormOneDoCommand (event->data.menu.itemID);

    } else if (event->eType == frmUpdateEvent) {

	FrmDrawForm (FrmGetActiveForm());
	DrawGameBoard();
	handled = true;

    } else if (event->eType == frmOpenEvent) {

	frm = FrmGetActiveForm();
	FrmDrawForm(frm);
	DrawGameBoard();
	handled = true;

    } else if (event->eType == frmCloseEvent) {

    }

    return (handled);
}


/***********************************************************************
 *
 * FUNCTION:    MainFormTwoHandleEvent
 *
 ***********************************************************************/

static Boolean MainFormTwoHandleEvent (EventPtr event)
{
    FormPtr frm;
    Boolean handled = false;
    UInt16 i;

    if (event->eType == ctlSelectEvent) {
	
	switch (event->data.ctlSelect.controlID) {

	case MainFormTwoNewGameButton:

            Game.P2IsFirstMove = true;

	    NewGameBoard();

            if (Game.P2FirstMove == PLAYER_PIECE) {
	        Game.P2turn = COMPUTER_PIECE;
		Game.P2FirstMove = COMPUTER_PIECE;
	    } else {
	        Game.P2turn = PLAYER_PIECE;
		Game.P2FirstMove = PLAYER_PIECE;
	    }
	    
            DrawGameBoard();
	    break;
			
	default:
	    break;
	}

    } else if ( event->eType == penDownEvent ) {

	Boolean	inBounds;
				
	handled = HandlePenDown( event->screenX, event->screenY,
	((KeyCurrentState() & (keyBitPageUp | keyBitPageDown)) != 0), 
	&inBounds );
		
	if ( !handled && inBounds ) {

	    handled = true;
	}
		
    } else if (event->eType == menuEvent) {

	return MainFormTwoDoCommand (event->data.menu.itemID);

    } else if (event->eType == frmUpdateEvent) {

	FrmDrawForm(FrmGetActiveForm());
	DrawGameBoard();
	handled = true;

    } else if (event->eType == frmOpenEvent) {

	frm = FrmGetActiveForm();
	FrmDrawForm(frm);
	DrawGameBoard();
	handled = true;

    } else if (event->eType == frmCloseEvent) {

    }

    return (handled);
}


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 ***********************************************************************/

static Boolean AppHandleEvent( EventPtr eventP)
{
    UInt16 formId;
    FormPtr frmP;

    if ( eventP->eType == frmLoadEvent ) {

	// Load the form resource.

	formId = eventP->data.frmLoad.formID;
	frmP = FrmInitForm(formId);
	FrmSetActiveForm(frmP);

	// Set the event handler for the form.  The handler of the currently
	// active form is called by FrmHandleEvent each time is receives an
	// event.

	switch (formId) {

	case MainFormOne:
	    FrmSetEventHandler(frmP, MainFormOneHandleEvent);
	    break;

	case MainFormTwo:
	    FrmSetEventHandler(frmP, MainFormTwoHandleEvent);
	    break;

	default:
            ErrNonFatalDisplay("Invalid Form Load Event");
	    break;

	}

	return true;
    }
	
    return false;
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 ***********************************************************************/

static void AppEventLoop(void)
{
    UInt16 error;
    EventType event;


    do {

	EvtGetEvent(&event, evtWaitForever);
		
	// Swallow up/down arrow key chars to prevent the clicking sound
	// when the keys are being used as modifiers for marking cells
	
	if ( ( event.eType == keyDownEvent ) && 
	   ( !TxtCharIsHardKey( 
		event.data.keyDown.modifiers,
		event.data.keyDown.chr ) ) &&
	   ( EvtKeydownIsVirtual( &event ) ) &&
	   ( ( event.data.keyDown.chr == vchrPageUp ) ||
	   ( event.data.keyDown.chr == vchrPageDown ) ) ) 
	{

            ;		// Do nothing

	} else if ( !SysHandleEvent( &event ) )
	    if ( !MenuHandleEvent( 0, &event, &error ))
		if ( !AppHandleEvent( &event ) )
		    FrmDispatchEvent( &event );

	// Check the heaps after each event

	#if EMULATION_LEVEL != EMULATION_NONE
	    MemHeapCheck(0);
	    MemHeapCheck(1);
	#endif

    } while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 ***********************************************************************/
 
UInt32 PilotMain (UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    UInt16 error;

    if ( cmd == sysAppLaunchCmdNormalLaunch ) {

	error = StartApplication();
	if (error) 
	    return(error);

	AppEventLoop();
	StopApplication();
    }

    return(0);
}


/***********************************************************************
 *
 * FUNCTION:    play
 *
 ***********************************************************************/
 
SUCESSOR_PTR play(STATE_PTR p, UInt16 depth)
{
    SUCESSOR_PTR q, r;
    Int16 max, t;
    UInt16 i, j;
    
    r = New_SUCESSOR();
    r->num = 0;
    q = gen_sucessors(p);
    max = -30000;
    for (i = 0; i<q->num; ++i) {
	t = -f(q->ptr[i], -30000, 30000, depth-1);
	if (t==max) {
	    j = r->num;
	    r->ptr[j] = New_STATE();
	    copy_state(q->ptr[i], r->ptr[j]);
	    r->ptr[j]->piece = max;
	    ++(r->num);
	} else if (t>max) {
	    max = t;
	    for (j = 0; j<r->num; ++j) {
	        r->ptr[j]->memPtr = FreeStates;
		FreeStates = r->ptr[j];
            }
	    r->num = 1;
	    r->ptr[0] = New_STATE();
	    copy_state(q->ptr[i], r->ptr[0]);
	    r->ptr[0]->piece = max;
	}
    }
    free_sucessors(q);
    return(r);
}


/***********************************************************************
 *
 * FUNCTION:    f
 *
 ***********************************************************************/
 
Int16 f(STATE_PTR p, Int16 alpha, Int16 beta, UInt16 depth)
{
    SUCESSOR_PTR q;
    Int16 t;
    UInt16 i;
    
    if (depth==0)
	return(value(p));
    q = gen_sucessors(p);
    if (q->num==0)
	alpha = value(p);
    for (i = 0; i<q->num && alpha<beta; ++i) {
	t = -f(q->ptr[i], -beta, -alpha, depth-1);
	if (t>alpha)
	    alpha = t;
    }
    free_sucessors(q);
    return(alpha);
}


/***********************************************************************
 *
 * FUNCTION:    gen_sucessors
 *
 ***********************************************************************/

SUCESSOR_PTR gen_sucessors(STATE_PTR p)
{
    SUCESSOR_PTR q;
    STATE_PTR newp;
    UInt16 i;
    
    q = New_SUCESSOR();
    q->num = 0;
	
    if (value(p) == 0) {
	for (i = 0; i<9; ++i) {
	    if (p->board[i] == NO_PIECE) {
		newp = New_STATE();
		copy_state(p, newp);
		newp->board[i] = p->turn;
		if (p->turn == COMPUTER_PIECE)
		    newp->turn = PLAYER_PIECE;
		else
		    newp->turn = COMPUTER_PIECE;
		q->ptr[q->num] = newp;
		++(q->num);
	    }
	}
    }
    return(q);
}    


/***********************************************************************
 *
 * FUNCTION:    free_sucessors
 *
 ***********************************************************************/

void free_sucessors(SUCESSOR_PTR q)
{
    UInt16 i;
    
    for (i = 0; i<q->num; ++i) {
        q->ptr[i]->memPtr = FreeStates;
	FreeStates = q->ptr[i];
    }
    q->memPtr = FreeSucessors;
    FreeSucessors = q;
}	


/***********************************************************************
 *
 * FUNCTION:    value
 *
 ***********************************************************************/

Int16 value(STATE_PTR p)
{
    Int16 x;
    
    x = 0;
	
    if (((p->board[0] == PLAYER_PIECE) &&
	 (p->board[1] == PLAYER_PIECE) &&
	 (p->board[2] == PLAYER_PIECE)) ||

	((p->board[3] == PLAYER_PIECE) &&
	 (p->board[4] == PLAYER_PIECE) &&
	 (p->board[5] == PLAYER_PIECE)) ||

	((p->board[6] == PLAYER_PIECE) &&
	 (p->board[7] == PLAYER_PIECE) &&
	 (p->board[8] == PLAYER_PIECE)) ||
		
        ((p->board[0] == PLAYER_PIECE) &&
	 (p->board[3] == PLAYER_PIECE) &&
	 (p->board[6] == PLAYER_PIECE)) ||

	((p->board[1] == PLAYER_PIECE) &&
	 (p->board[4] == PLAYER_PIECE) &&
	 (p->board[7] == PLAYER_PIECE)) ||

	((p->board[2] == PLAYER_PIECE) &&
	 (p->board[5] == PLAYER_PIECE) &&
	 (p->board[8] == PLAYER_PIECE)) ||
		
        ((p->board[0] == PLAYER_PIECE) &&
	 (p->board[4] == PLAYER_PIECE) &&
	 (p->board[8] == PLAYER_PIECE)) ||

	((p->board[2] == PLAYER_PIECE) &&
	 (p->board[4] == PLAYER_PIECE) &&
	 (p->board[6] == PLAYER_PIECE)))
		
		x = 25000;
	
    else if
		
       (((p->board[0] == COMPUTER_PIECE) &&
	 (p->board[1] == COMPUTER_PIECE) &&
	 (p->board[2] == COMPUTER_PIECE)) ||

	((p->board[3] == COMPUTER_PIECE) &&
	 (p->board[4] == COMPUTER_PIECE) &&
	 (p->board[5] == COMPUTER_PIECE)) ||

	((p->board[6] == COMPUTER_PIECE) &&
	 (p->board[7] == COMPUTER_PIECE) &&
	 (p->board[8] == COMPUTER_PIECE)) ||
	
        ((p->board[0] == COMPUTER_PIECE) &&
	 (p->board[3] == COMPUTER_PIECE) &&
	 (p->board[6] == COMPUTER_PIECE)) ||

	((p->board[1] == COMPUTER_PIECE) &&
	 (p->board[4] == COMPUTER_PIECE) &&
	 (p->board[7] == COMPUTER_PIECE)) ||

	((p->board[2] == COMPUTER_PIECE) &&
	 (p->board[5] == COMPUTER_PIECE) &&
	 (p->board[8] == COMPUTER_PIECE)) ||
		
        ((p->board[0] == COMPUTER_PIECE) &&
	 (p->board[4] == COMPUTER_PIECE) &&
	 (p->board[8] == COMPUTER_PIECE)) ||

	((p->board[2] == COMPUTER_PIECE) &&
	 (p->board[4] == COMPUTER_PIECE) &&
	 (p->board[6] == COMPUTER_PIECE)))
	
		x = -25000;
	
    if (p->turn == COMPUTER_PIECE)
        x = -x;
    return(x);
}


/***********************************************************************
 *
 * FUNCTION:    New_STATE
 *
 ***********************************************************************/

STATE_PTR New_STATE(void)
{
    STATE_PTR p;
    MemHandle pH;
    
    if (FreeStates == NULL) {
        pH = MemHandleNew(sizeof(STATE));
        ErrFatalDisplayIf( !pH, "MemHandleNew() Returned 0, OUT OF RAM" );
	p = MemHandleLock(pH);
	p->memHandle = pH;
    } else {
	p = FreeStates;
        FreeStates = FreeStates->memPtr;
    }
    return(p);
}


/***********************************************************************
 *
 * FUNCTION:    New_SUCESSOR
 *
 ***********************************************************************/

SUCESSOR_PTR New_SUCESSOR(void)
{
    SUCESSOR_PTR q;
    MemHandle qH;
    
    if (FreeSucessors == NULL) {
        qH = MemHandleNew(sizeof(SUCESSOR));
        ErrFatalDisplayIf( !qH, "MemHandleNew() Returned 0, OUT OF RAM" );
	q = MemHandleLock(qH);
	q->memHandle = qH;
    } else {
	q = FreeSucessors;
        FreeSucessors = FreeSucessors->memPtr;
    }
    return(q);
}


/***********************************************************************
 *
 * FUNCTION:    copy_state
 *
 ***********************************************************************/

void copy_state(STATE_PTR p, STATE_PTR q)
{
    Int16 i;

    for (i = 0; i<9; ++i)
	q->board[i] = p->board[i];
    q->turn = p->turn;
    q->piece = p->piece;
}


/***********************************************************************
 *
 * FUNCTION:	ConvertSquare
 *
 ***********************************************************************/

static Int16 ConvertSquare(UInt16 bitmap)
{
    if (Game.numPlayers != 2) {
    
        if (!Game.P1reversed) {

            switch( bitmap) {

            case P1EmptyBitmap:
                return(NO_PIECE);

            case P1CrossBitmap:
                return(PLAYER_PIECE);

            case P1NoughtBitmap:
                return(COMPUTER_PIECE);
            }
        } else {

            switch( bitmap) {

            case P1EmptyBitmap:
                return(NO_PIECE);

            case P1CrossBitmap:
                return(COMPUTER_PIECE);

            case P1NoughtBitmap:
                return(PLAYER_PIECE);
            }
        }

    } else {

        switch( bitmap) {

        case P2EmptyBitmap:
            return(NO_PIECE);

        case P2CrossBitmap:
            return(PLAYER_PIECE);

        case P2NoughtBitmap:
            return(COMPUTER_PIECE);
        }
    }

    ErrFatalDisplayIf( 1, "illegal bitmap" );
}


/***********************************************************************
 *
 * FUNCTION:	ConvertBitmap
 *
 ***********************************************************************/

static Int16 ConvertBitmap(UInt16 piece)
{
    if (Game.numPlayers != 2) {
    
        if (!Game.P1reversed) {
    
            switch( piece ) {

            case NO_PIECE:
                return(P1EmptyBitmap);

            case PLAYER_PIECE:
                return(P1CrossBitmap);

            case COMPUTER_PIECE:
                return(P1NoughtBitmap);
 	    }
        } else {
    
            switch( piece ) {

            case NO_PIECE:
                return(P1EmptyBitmap);

            case PLAYER_PIECE:
                return(P1NoughtBitmap);

            case COMPUTER_PIECE:
                return(P1CrossBitmap);
 	    }
        }
    } else {

        switch( piece ) {

        case NO_PIECE:
            return(P2EmptyBitmap);

        case PLAYER_PIECE:
            return(P2CrossBitmap);

        case COMPUTER_PIECE:
            return(P2NoughtBitmap);
 	}
    }

    ErrFatalDisplayIf( 1, "illegal piece" );
}


/***********************************************************************
 *
 * FUNCTION:    CaptureGraphics
 *
 ***********************************************************************/

static void CaptureGraphics( void )
{
    Int16 i;
    
    if (Game.numPlayers != 2)
        for ( i = 0; i < 9; ++i )
	    move.board[i] = ConvertSquare( Game.P1square[i] );
    else
        for ( i = 0; i < 9; ++i )
	    move.board[i] = ConvertSquare( Game.P2square[i] );

}


/***********************************************************************
 *
 * FUNCTION:    RestoreGraphics
 *
 ***********************************************************************/

static void RestoreGraphics( void )
{
    Int16 i;
    
    if (Game.numPlayers != 2)
        for ( i = 0; i < 9; ++i )
	    Game.P1square[i] = ConvertBitmap( move.board[i]);
    else
        for ( i = 0; i < 9; ++i )
	    Game.P2square[i] = ConvertBitmap( move.board[i]);
}


/***********************************************************************
 *
 * FUNCTION:    DrawGrid
 *
 ***********************************************************************/

static void DrawGrid( void )
{
    Int16	x1, y1, x2, y2;
    
    x1 = BoardOffsetX + BoardSquareSize;
    y1 = BoardOffsetY;
    x2 = x1;
    y2 = y1 + BoardSquareSize;
    WinDrawLine( x1, y1, x2, y2 );
    
    x1 = x2 - BoardSquareSize;
    y1 = y2;
    WinDrawLine( x1, y1, x2, y2 );
    
    x2 = x1;
    y2 = y1 + BoardGridWidth;
    WinDrawLine( x1, y1, x2, y2 );

    x1 = x2 + BoardSquareSize;
    y1 = y2;
    WinDrawLine( x1, y1, x2, y2 );
    
    x2 = x1;
    y2 = y1 + BoardSquareSize;
    WinDrawLine( x1, y1, x2, y2 );

    x1 = x2 - BoardSquareSize;
    y1 = y2;
    WinDrawLine( x1, y1, x2, y2 );
    
    x2 = x1;
    y2 = y1 + BoardGridWidth;
    WinDrawLine( x1, y1, x2, y2 );

    x1 = x2 + BoardSquareSize;
    y1 = y2;
    WinDrawLine( x1, y1, x2, y2 );
    
    x2 = x1;
    y2 = y1 + BoardSquareSize;
    WinDrawLine( x1, y1, x2, y2 );

    x1 = x2 + BoardGridWidth;
    y1 = y2;
    WinDrawLine( x1, y1, x2, y2 );

    x2 = x1;
    y2 = y1 - BoardSquareSize;
    WinDrawLine( x1, y1, x2, y2 );

    x1 = x2 + BoardSquareSize;
    y1 = y2;
    WinDrawLine( x1, y1, x2, y2 );

    x2 = x1;
    y2 = y1 + BoardSquareSize;
    WinDrawLine( x1, y1, x2, y2 );
    
    x1 = x2 + BoardGridWidth;
    y1 = y2;    
    WinDrawLine( x1, y1, x2, y2 );
    
    x2 = x1;
    y2 = y1 - BoardSquareSize;
    WinDrawLine( x1, y1, x2, y2 );

    x1 = x2 + BoardSquareSize;
    y1 = y2;
    WinDrawLine( x1, y1, x2, y2 );

    x2 = x1;
    y2 = y1 - BoardGridWidth;
    WinDrawLine( x1, y1, x2, y2 );

    x1 = x2 - BoardSquareSize;
    y1 = y2;
    WinDrawLine( x1, y1, x2, y2 );

    x2 = x1;
    y2 = y1 - BoardSquareSize;
    WinDrawLine( x1, y1, x2, y2 );

    x1 = x2 + BoardSquareSize;
    y1 = y2;
    WinDrawLine( x1, y1, x2, y2 );

    x2 = x1;
    y2 = y1 - BoardGridWidth;
    WinDrawLine( x1, y1, x2, y2 );

    x1 = x2 - BoardSquareSize;
    y1 = y2;
    WinDrawLine( x1, y1, x2, y2 );
    
    x2 = x1;
    y2 = y1 - BoardSquareSize;
    WinDrawLine( x1, y1, x2, y2 );
    
    x1 = x2 - BoardGridWidth;
    y1 = y2;    
    WinDrawLine( x1, y1, x2, y2 );

    x2 = x1;
    y2 = y1 + BoardSquareSize;    
    WinDrawLine( x1, y1, x2, y2 );

    x1 = x2 - BoardSquareSize;
    y1 = y2;
    WinDrawLine( x1, y1, x2, y2 );

    x2 = x1;
    y2 = y1 - BoardSquareSize;
    WinDrawLine( x1, y1, x2, y2 );

    x1 = x2 - BoardGridWidth;
    y1 = y2;
    WinDrawLine( x1, y1, x2, y2 );

    x1 = BoardOffsetX + BoardSquareSize + BoardGridWidth;
    y1 = BoardOffsetY + BoardSquareSize + BoardGridWidth;
    x2 = x1;
    y2 = y1 + BoardSquareSize;
    WinDrawLine( x1, y1, x2, y2 );

    x1 = x2 + BoardSquareSize;
    y1 = y2;
    WinDrawLine( x1, y1, x2, y2 );

    x2 = x1;
    y2 = y1 - BoardSquareSize;
    WinDrawLine( x1, y1, x2, y2 );

    x1 = x2 - BoardSquareSize;
    y1 = y2;
    WinDrawLine( x1, y1, x2, y2 );

}



